# RegenMod  release 5 by David Bowland
# ./addons/eventscripts/regenmod/regenmod.py

# >>> To configure this addon please see regenmod.cfg <<<

"""
Gives players health and armor based on damage, weapon, and hitbox; sets spawn health and money; awards monetary bonuses for high-damage hits. Requires only ES 2.0+
"""


import es
import langlib
import os.path
import playerlib
import gamethread
import time


info = es.AddonInfo()
info.name = 'RegenMod'
info.version = '5'
info.url = 'http://addons.eventscripts.com/addons/view/regenmod'
info.basename = 'regenmod'
info.author = 'SuperDave'


dict_weapons = {}
dict_weapons['deagle'] = {'prop':'CBasePlayer.localdata.m_iAmmo.001', 'maxammo':'50AE', 'slot':'secondary', 'clip':7}
dict_weapons['ak47'] = {'prop':'CBasePlayer.localdata.m_iAmmo.002', 'maxammo':'762mm', 'slot':'primary', 'clip':30}
dict_weapons['scout'] = {'prop':'CBasePlayer.localdata.m_iAmmo.002', 'maxammo':'762mm', 'slot':'primary', 'clip':10}
dict_weapons['aug'] = {'prop':'CBasePlayer.localdata.m_iAmmo.002', 'maxammo':'762mm', 'slot':'primary', 'clip':30}
dict_weapons['g3sg1'] = {'prop':'CBasePlayer.localdata.m_iAmmo.002', 'maxammo':'762mm', 'slot':'primary', 'clip':20}
dict_weapons['galil'] = {'prop':'CBasePlayer.localdata.m_iAmmo.003', 'maxammo':'556mm', 'slot':'primary', 'clip':35}
dict_weapons['famas'] = {'prop':'CBasePlayer.localdata.m_iAmmo.003', 'maxammo':'556mm', 'slot':'primary', 'clip':25}
dict_weapons['m4a1'] = {'prop':'CBasePlayer.localdata.m_iAmmo.003', 'maxammo':'556mm', 'slot':'primary', 'clip':30}
dict_weapons['sg552'] = {'prop':'CBasePlayer.localdata.m_iAmmo.003', 'maxammo':'556mm', 'slot':'primary', 'clip':30}
dict_weapons['sg550'] = {'prop':'CBasePlayer.localdata.m_iAmmo.003', 'maxammo':'556mm', 'slot':'primary', 'clip':30}
dict_weapons['m249'] = {'prop':'CBasePlayer.localdata.m_iAmmo.004', 'maxammo':'556mm_box', 'slot':'primary', 'clip':100}
dict_weapons['awp'] = {'prop':'CBasePlayer.localdata.m_iAmmo.005', 'maxammo':'338mag', 'slot':'primary', 'clip':10}
dict_weapons['tmp'] = {'prop':'CBasePlayer.localdata.m_iAmmo.006', 'maxammo':'9mm', 'slot':'primary', 'clip':30}
dict_weapons['mp5navy'] = {'prop':'CBasePlayer.localdata.m_iAmmo.006', 'maxammo':'9mm', 'slot':'primary', 'clip':30}
dict_weapons['glock'] = {'prop':'CBasePlayer.localdata.m_iAmmo.006', 'maxammo':'9mm', 'slot':'secondary', 'clip':20}
dict_weapons['elite'] = {'prop':'CBasePlayer.localdata.m_iAmmo.006', 'maxammo':'9mm', 'slot':'secondary', 'clip':30}
dict_weapons['m3'] = {'prop':'CBasePlayer.localdata.m_iAmmo.007', 'maxammo':'buckshot', 'slot':'primary', 'clip':8}
dict_weapons['xm1014'] = {'prop':'CBasePlayer.localdata.m_iAmmo.007', 'maxammo':'buckshot', 'slot':'primary', 'clip':7}
dict_weapons['mac10'] = {'prop':'CBasePlayer.localdata.m_iAmmo.008', 'maxammo':'45acp', 'slot':'primary', 'clip':30}
dict_weapons['ump45'] = {'prop':'CBasePlayer.localdata.m_iAmmo.008', 'maxammo':'45acp', 'slot':'primary', 'clip':25}
dict_weapons['usp'] = {'prop':'CBasePlayer.localdata.m_iAmmo.008', 'maxammo':'45acp', 'slot':'secondary', 'clip':12}
dict_weapons['p228'] = {'prop':'CBasePlayer.localdata.m_iAmmo.009', 'maxammo':'357sig', 'slot':'secondary', 'clip':13}
dict_weapons['fiveseven'] = {'prop':'CBasePlayer.localdata.m_iAmmo.010', 'maxammo':'57mm', 'slot':'secondary', 'clip':20}
dict_weapons['p90'] = {'prop':'CBasePlayer.localdata.m_iAmmo.010', 'maxammo':'57mm', 'slot':'primary', 'clip':50}
dict_weapons['knife'] = {'prop':'none', 'maxammo':'none', 'slot':'knife', 'clip':0}
dict_weapons['hegrenade'] = {'prop':'CBasePlayer.localdata.m_iAmmo.011', 'maxammo':'hegrenade', 'slot':'grenade', 'clip':0}
dict_weapons['flashbang'] = {'prop':'CBasePlayer.localdata.m_iAmmo.012', 'maxammo':'flashbang', 'slot':'grenade', 'clip':0}
dict_weapons['smokegrenade'] = {'prop':'CBasePlayer.localdata.m_iAmmo.013', 'maxammo':'smokegrenade', 'slot':'grenade', 'clip':0}

tuple_hitgroup_strings = ('kill', 'head', 'chest', 'stomach', 'arm', 'arm', 'leg', 'leg')

dict_players = {}
dict_player_vars = {'int_rounds_played':0, 'int_damage_given':0, 'int_last_round_damage':0, 'int_health_conserved':0, 'int_kills':0, 'int_deaths':0, 'float_spawn_score':0, 'int_hedamaged_players':0, 'float_spawn_time':0, 'float_time_alive':0}
int_blinded_players = 0

dict_weapon_regen = {}
dict_options = {'regenmod_regeneration_maxhealth':[150, 'Maximum health players can attain with regneration'], 'regenmod_regeneration_maxarmor':[75, 'Maximum armor players can attain with regneration'], 'regenmod_regeneration_cankill':[1, '0 = players have a lower limit of one when gaining negative health, 1 = players gaining negative health can die'], 'regenmod_regeneration_killpercent':[150, 'Percent of damage to regenerate when an opponent is killed'], 'regenmod_regeneration_headpercent':[180, 'Percent of damage to regenerate when an opponent is hurt in the head'], 'regenmod_regeneration_chestpercent':[100, 'Percent of damage to regenerate when an opponent is hurt in the chest'], 'regenmod_regeneration_stomachpercent':[90, 'Percent of damage to regenerate when an opponent is hurt in the stomach'], 'regenmod_regeneration_armpercent':[75, 'Percent of damage to regenerate when an opponent is hurt in the arm'], 'regenmod_regeneration_legpercent':[60, 'Percent of damage to regenerate when an opponent is hurt in the leg'], 'regenmod_regeneration_teampercent':[-200, 'Percent of damage to regenerate when the victim is a teammate'], 'regenmod_regeneration_primarypercent':[10, 'Percent of damage to regenerate when the victim is hurt with a primary weapon'], 'regenmod_regeneration_secondarypercent':[25, 'Percent of damage to regenerate when the victim is hurt with a secondary weapon'], 'regenmod_regeneration_knifepercent':[85, 'Percent of damage to regenerate when the victim is hurt with a knife'], 'regenmod_regeneration_grenadepercent':[65, 'Percent of damage to regenerate when the victim is hurt with a grenade'], 'regenmod_ammunition_primary':[1, '0 = no change, 1 = regenerate, 2 = infinite'], 'regenmod_ammunition_secondary':[2, '0 = no change, 1 = regenerate, 2 = infinite'], 'regenmod_ammunition_hegrenades':[1, '0 = no change, 1 = regenerate, 2 = infinite'], 'regenmod_ammunition_flashbangs':[1, '0 = no change, 1 = regenerate, 2 = infinite'], 'regenmod_ammunition_smokegrenades':[0, '0 = no change, 1 = infinite'], 'regenmod_ammunition_regenovermax':[0, '0 = ammunition is restricted to the maximum amount, 1 = no restriction on the amount of ammunition'], 'regenmod_spawn_health':[1, '0 = no change, 1 = set player health at spawn'], 'regenmod_spawn_health_max':[200, 'Maximum amount of health players will receive at spawn when applicable'], 'regenmod_spawn_health_min':[60, 'Minimum amount of heath players will receive at spawn when applicable'], 'regenmod_spawn_money':[1, '0 = no change, 1 = set player money at spawn'], 'regenmod_spawn_money_max':[8000, 'Maximum amount of money players will receive at spawn when applicable'], 'regenmod_spawn_money_min':[2000, 'Minimum amount of money players will receive at spawn when applicable'], 'regenmod_spawn_damagegiven_weight':[40, 'Weight given to damage given when computing spawn health and money'], 'regenmod_spawn_healthconserved_weight':[20, 'Weight given to health conserved when computing spawn health and money'], 'regenmod_spawn_kills_weight':[20, 'Weight given to kills when computing spawn health and money'], 'regenmod_spawn_deaths_weight':[10, 'Weight given to deaths when computing spawn health and money'], 'regenmod_spawn_time_weight':[10, 'Weight given to time alive when computing spawn health and money'], 'regenmod_damagebonus_threshold':[75, 'Damage a player must cause in one hit to receive a damage bonus--set to 0 for no damage bonus'], 'regenmod_damagebonus_base':[50, 'Amount of money players receive for a hit above the damage bonus threshold'], 'regenmod_damagebonus_increment':[50, 'Damage increments for the damage bonus multiplier'], 'regenmod_announce_mostdamage_center':[1, 'Most damage message: 0 = no center text, 1 = center text'], 'regenmod_announce_mostdamage_text':[1, 'Most damage message: 0 = no console or chat text, 1 = console text only, 2 = chat text and console text'], 'regenmod_announce_damagebonus_chat':[1, 'Damage bonus message: 0 = no chat text, 1 = only for player receiving bonus, 2 = for all players'], 'regenmod_announce_damagebonus_console':[0, 'Damage bonus message:  0 = no console text, 1 = only for player receiving bonus, 2 = for all players'], 'regenmod_announce_damagebonus_hud':[1, 'Damage bonus message:  0 = no HUD text, 1 = HUD text'], 'regenmod_announce_spawnstats_text':[1, 'Spawn statistics message:  0 = no console or chat text, 1 = console text only, 2 = chat text and console text'], 'regenmod_announce_spawnstats_hud':[1, 'Spawn statistics message:  0 = no HUD text, 1 = HUD text'], 'regenmod_announce_spawnstats_all_console':[1, 'Spawn statistics of all players message:  0 = no console text, 1 = console text']}
func_lang_text = None


def load():
   """
   Creates commands for settings
   Checks to make sure critical server variables are created by the config
   Creates a dictionary for each active player in dict_players
   """
   global func_lang_text

   if not es.exists('command', 'regenmod_regeneration_add'):
      es.regcmd('regenmod_regeneration_add', 'regenmod/weapon_specific_cmd', 'regenmod_regeneration_add <weapon> <regeneration percent>\nServer command for designating health regeneration percent for specific weapons')
   if not es.exists('command', 'regenmod_regeneration_remove'):
      es.regcmd('regenmod_regeneration_remove', 'regenmod/weapon_specific_cmd', 'regenmod_regeneration_remove <weapon>\nServer command for removing health regeneration percent for specific weapons')

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('regenmod') + '/regenmod.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/regenmod/regenmod.cfg')
   else:
      es.dbgmsg(0, 'RegenMod: Unable to load regenmod.cfg! Please ensure it is in the ./regenmod/ directory.')

   str_lang_path = es.getAddonPath('regenmod') + '/regenmod_languages.ini'
   if os.path.isfile(str_lang_path):
      func_lang_text = langlib.Strings(str_lang_path)
   else:
      es.dbgmsg(0, 'RegenMod: Unable to load regenmod_languages.ini! Please ensure it is in the ./regenmod/ directory.')
   if not func_lang_text:
      func_lang_text = missing_lang

   for int_current_id in es.getUseridList():
      if es.getplayerteam(int_current_id) > 1:
         dict_players[str(int_current_id)] = dict_player_vars.copy()
         if not es.getplayerprop(int_current_id, 'CBasePlayer.pl.deadflag'):
            dict_players[str(int_current_id)]['float_spawn_time'] = time.time()


def es_map_start(event_var):
   """Clears the dictionary of players so each player is recreated on the new map"""
   dict_players.clear()


def round_start(event_var):
   """
   Resets the number of blinded players to zero
   Loops through players to find the most damage for the last round
   Displays the player with the most damage last round
   """
   global int_blinded_players
   int_blinded_players = 0

   int_most_damage = 0
   for str_userid in dict_players:
      int_loop_damage = dict_players[str_userid]['int_last_round_damage']
      if int_loop_damage > int_most_damage:
         int_most_damage = int_loop_damage
         str_most_damage_player = es.getplayername(str_userid)
      dict_players[str_userid]['int_last_round_damage'] = 0

   if int_most_damage:
      dict_lang_tokens = {'player':str_most_damage_player, 'damage':int_most_damage}
      int_option_center = int(es.ServerVar('regenmod_announce_mostdamage_center'))
      int_option_text = int(es.ServerVar('regenmod_announce_mostdamage_text'))
      for int_userid in es.getUseridList():
         str_player_lang = playerlib.getPlayer(int_userid).get('lang')
         if int_option_center:
            es.centertell(int_userid, remove_tags(func_lang_text('most damage', dict_lang_tokens, str_player_lang)))
         if int_option_text > 1:
            es.tell(int_userid, '#multi', func_lang_text('most damage', dict_lang_tokens, str_player_lang))
         elif int_option_text:
            es.cexec(int_userid, 'echo ' + remove_tags(func_lang_text('most damage', dict_lang_tokens, str_player_lang)))


def round_end(event_var):
   """Adds remaining health to each live player's health conserved"""
   global dict_players

   if event_var['reason'] <> '16':
      float_current_time = time.time()
      for str_current_id in dict_players:
         if es.getplayerteam(str_current_id) > 1:
            dict_current_player = dict_players[str_current_id]
            dict_current_player['int_rounds_played'] += 1
            if not es.getplayerprop(str_current_id, 'CBasePlayer.pl.deadflag'):
               dict_current_player['int_health_conserved'] += es.getplayerprop(str_current_id, 'CBasePlayer.m_iHealth')
            if dict_current_player['float_spawn_time']:
               dict_current_player['float_time_alive'] += float_current_time - dict_current_player['float_spawn_time']
               dict_current_player['float_spawn_time'] = float_current_time


def player_spawn(event_var):
   """
   Creates the spawning player in the dictionary
   Displays player statistics
   Creates sorted lists of all players' statistics
   Creates a list of scores based on player rank
   Sets players health and money as necessary
   """
   global dict_players

   if int(event_var['es_userteam']) > 1:
      str_userid = event_var['userid']
      if not dict_players.has_key(str_userid):
         dict_players[str_userid] = dict_player_vars.copy()

      dict_current_player = dict_players[str_userid]
      dict_current_player['int_hedamaged_players'] = 0
      str_player_lang = playerlib.getPlayer(str_userid).get('lang')
      dict_kd = {'kills':dict_current_player['int_kills'], 'deaths':dict_current_player['int_deaths'], 'time':format_time(dict_current_player['float_time_alive'])}
      dict_dh = {'damage':dict_current_player['int_damage_given'], 'health':dict_current_player['int_health_conserved']}
      int_spawnstats_text = int(es.ServerVar('regenmod_announce_spawnstats_text'))
      if int_spawnstats_text > 1:
         es.tell(str_userid, '#multi', func_lang_text('kills deaths', dict_kd, str_player_lang))
         es.tell(str_userid, '#multi', func_lang_text('damage health', dict_dh, str_player_lang))
      elif int_spawnstats_text:
         es.cexec(str_userid, 'echo \"%s\"' % remove_tags(func_lang_text('kills deaths', dict_kd, str_player_lang)))
         es.cexec(str_userid, 'echo \"%s\"' % remove_tags(func_lang_text('damage health', dict_dh, str_player_lang)))
      if int(es.ServerVar('regenmod_announce_spawnstats_hud')):
         str_text_1 = remove_tags(func_lang_text('kills deaths', dict_kd, str_player_lang))
         str_text_2 = remove_tags(func_lang_text('damage health', dict_dh, str_player_lang))
         int_text_len = len(str_text_1)
         int_temp_len = len(str_text_2)
         if int_temp_len > int_text_len:
            int_text_len = int_temp_len
         gamethread.delayed(1, hud_hint, (str_userid, str_text_1.center(int_temp_len) + '\n' + str_text_2.center(int_temp_len)))

      list_spawn_score = []
      dict_loop_lists = {'int_damage_given':[float(es.ServerVar('regenmod_spawn_damagegiven_weight')), []], 'int_health_conserved':[float(es.ServerVar('regenmod_spawn_healthconserved_weight')), []], 'int_kills':[float(es.ServerVar('regenmod_spawn_kills_weight')), []], 'int_deaths':[float(es.ServerVar('regenmod_spawn_deaths_weight')), []], 'float_time_alive':[float(es.ServerVar('regenmod_spawn_time_weight')), []]}
      int_spawnstats_all_console = int(es.ServerVar('regenmod_announce_spawnstats_all_console'))
      for str_player in dict_players:
         dict_loop_player = dict_players[str_player]
         if dict_loop_player['int_rounds_played']:
            for str_property in dict_loop_lists:
               dict_loop_lists[str_property][1].append(dict_loop_player[str_property] / dict_loop_player['int_rounds_played'])
            if int_spawnstats_all_console and str_player <> str_userid:
               es.cexec(str_userid, 'echo \"%s\"' % func_lang_text('stats message', {'player':es.getplayername(str_player)}, str_player_lang))
               es.cexec(str_userid, 'echo \"%s\"' % remove_tags(func_lang_text('kills deaths', {'kills':dict_loop_player['int_kills'], 'deaths':dict_loop_player['int_deaths'], 'time':format_time(dict_loop_player['float_time_alive'])}, str_player_lang)))
               es.cexec(str_userid, 'echo \"%s\"' % remove_tags(func_lang_text('damage health', {'damage':dict_loop_player['int_damage_given'], 'health':dict_loop_player['int_health_conserved']}, str_player_lang)))
         else:
            for str_property in dict_loop_lists:
               dict_loop_lists[str_property][1].append(0)

      for str_property in dict_loop_lists:
         dict_loop_lists[str_property][1].sort()
      dict_loop_lists['int_deaths'][1].reverse()

      for str_player in dict_players:
         if dict_players[str_player]['int_rounds_played']:
            float_temp_score = 0
            float_total_weight = 0
            for str_property in dict_loop_lists:
               float_total_weight += dict_loop_lists[str_property][0]
               float_temp_score += (dict_loop_lists[str_property][1].index(dict_players[str_player][str_property] / dict_players[str_player]['int_rounds_played']) * dict_loop_lists[str_property][0])
            if float_total_weight:
               dict_players[str_player]['float_spawn_score'] = float_temp_score / float_total_weight
            else:
               dict_players[str_player]['float_spawn_score'] = 0
         else:
            dict_players[str_player]['float_spawn_score'] = 0
         list_spawn_score.append(dict_players[str_player]['float_spawn_score'])

      list_spawn_score.sort()
      list_spawn_score.reverse()

      if int(es.ServerVar('regenmod_spawn_health')):
         if list_spawn_score[len(list_spawn_score) - 1] - list_spawn_score[0]:
            es.setplayerprop(str_userid, 'CBasePlayer.m_iHealth', int(list_spawn_score.index(dict_current_player['float_spawn_score']) * (int(es.ServerVar('regenmod_spawn_health_max')) - int(es.ServerVar('regenmod_spawn_health_min'))) / (len(list_spawn_score) - 1)) + int(es.ServerVar('regenmod_spawn_health_min')))

      if int(es.ServerVar('regenmod_spawn_money')):
         if len(list_spawn_score) > 1:
            es.setplayerprop(str_userid, 'CCSPlayer.m_iAccount', int(list_spawn_score.index(dict_current_player['float_spawn_score']) * (int(es.ServerVar('regenmod_spawn_money_max')) - int(es.ServerVar('regenmod_spawn_money_min'))) / (len(list_spawn_score) - 1)) + int(es.ServerVar('regenmod_spawn_money_min')))
         else:
            es.setplayerprop(str_userid, 'CCSPlayer.m_iAccount', es.ServerVar('regenmod_spawn_money_min'))

      dict_current_player['float_spawn_time'] = time.time()


def weapon_reload(event_var):
   """Gives the player full ammo for infinite ammunition"""
   str_weapon = event_var['es_userweapon'].replace('weapon_', '')
   if dict_weapons.has_key(str_weapon):
      dict_current_weapon = dict_weapons[str_weapon]
      if (dict_current_weapon['slot'] == 'primary') or (dict_current_weapon['slot'] == 'secondary'):
         if es.ServerVar('regenmod_ammunition_%s' % dict_current_weapon['slot']) > 1:
            str_userid = event_var['userid']
            int_bullets_in_clip = 0
            int_player_handle = es.getplayerhandle(str_userid)
            for int_index in es.createentitylist('weapon_' + str_weapon):
               if es.getindexprop(int_index, 'CBaseEntity.m_hOwnerEntity') == int_player_handle:
                  int_bullets_in_clip = es.getindexprop(int_index, 'CBaseCombatWeapon.LocalWeaponData.m_iClip1')
                  break
            es.setplayerprop(str_userid, dict_current_weapon['prop'], int(es.ServerVar('ammo_%s_max' % dict_current_weapon['maxammo'])) + dict_current_weapon['clip'] - int_bullets_in_clip)


def weapon_fire_on_empty(event_var):
   """Calls weapon_reload"""
   weapon_reload(event_var)


def player_hurt(event_var):
   """
   Determines int_weapon_percent based on the weapon used
   Gives the attacker ammunition as necessary, increments damage statistics, deermines int_hitgroup_percent
   Calculates and awards damage bonus
   Applies regeneration to health and armor
   """

   str_attacker_id = event_var['attacker']
   if int(str_attacker_id) and (event_var['userid'] <> str_attacker_id):
      str_weapon = event_var['weapon']
      dict_current_player = dict_players[str_attacker_id]
      if dict_weapon_regen.has_key(str_weapon):
         int_weapon_percent = dict_weapon_regen[str_weapon]
      elif dict_weapons.has_key(str_weapon):
         int_weapon_percent = int(es.ServerVar('regenmod_regeneration_%spercent' % dict_weapons[str_weapon]['slot']))
      else:
         int_weapon_percent = 0

      if event_var['es_userteam'] == event_var['es_attackerteam']:
         int_hitgroup_percent = int(es.ServerVar('regenmod_regeneration_teampercent'))
         if str_weapon == 'hegrenade':
            dict_current_player['int_hedamaged_players'] -= 1
      else:
         int_hitgroup_percent = int(es.ServerVar('regenmod_regeneration_%spercent' % tuple_hitgroup_strings[int(event_var['hitgroup'])]))
         dict_current_player['int_damage_given'] += int(event_var['dmg_health'])
         dict_current_player['int_last_round_damage'] += int(event_var['dmg_health'])
         if dict_weapons.has_key(str_weapon):
            dict_current_weapon = dict_weapons[str_weapon]
            if (dict_current_weapon['slot'] == 'primary') or (dict_current_weapon['slot'] == 'secondary'):
               int_ammo_option = int(es.ServerVar('regenmod_ammunition_%s' % dict_current_weapon['slot']))
               if int_ammo_option > 1:
                  int_ammo_amount = int(es.ServerVar('ammo_%s_max' % dict_current_weapon['maxammo']))
               elif int_ammo_option:
                  int_ammo_amount = es.getplayerprop(str_attacker_id, dict_current_weapon['prop']) + 1
                  int_max_ammo = int(es.ServerVar('ammo_%s_max' % dict_current_weapon['maxammo']))
                  if int_ammo_amount > int_max_ammo and not int(es.ServerVar('regenmod_ammunition_regenovermax')):
                     int_ammo_amount = int_max_ammo
               es.setplayerprop(str_attacker_id, dict_current_weapon['prop'], int_ammo_amount)
         if str_weapon == 'hegrenade':
            dict_current_player['int_hedamaged_players'] += 1

         int_damagebonus_difference = int(event_var['dmg_health']) - int(es.ServerVar('regenmod_damagebonus_threshold'))
         if (int_damagebonus_difference > 0) and int(es.ServerVar('regenmod_damagebonus_threshold')):
            int_player_money = int(es.ServerVar('regenmod_damagebonus_base')) + (int_damagebonus_difference * ((int_damagebonus_difference / int(es.ServerVar('regenmod_damagebonus_base'))) + 1))
            dict_lang_tokens = {'attacker':event_var['es_attackername'], 'damage':event_var['dmg_health'], 'victim':event_var['es_username'], 'bonus':int_player_money}
            str_attacker_lang = playerlib.getPlayer(str_attacker_id).get('lang')
            int_announce_option = int(es.ServerVar('regenmod_announce_damagebonus_chat'))
            if int_announce_option > 1:
               for int_userid in es.getUseridList():
                  es.tell(int_userid, '#multi', func_lang_text('damage bonus', dict_lang_tokens, playerlib.getPlayer(int_userid).get('lang')))
            elif int_announce_option:
               es.tell(str_attacker_id, '#multi', func_lang_text('player damage bonus', dict_lang_tokens, str_attacker_lang))
            int_announce_option = int(es.ServerVar('regenmod_announce_damagebonus_console'))
            if int_announce_option > 1:
               for int_userid in es.getUseridList():
                  es.cexec(int_userid, 'echo ' + remove_tags(func_lang_text('damage bonus', dict_lang_tokens, playerlib.getPlayer(int_userid).get('lang'))))
            elif int_announce_option:
               es.cexec(str_attacker_id, 'echo ' + remove_tags(func_lang_text('player damage bonus', dict_lang_tokens, str_attacker_lang)))
            if int(es.ServerVar('regenmod_announce_damagebonus_hud')):
               hud_hint(str_attacker_id, remove_tags(func_lang_text('player damage bonus', dict_lang_tokens, str_attacker_lang)))
            int_player_money += es.getplayerprop(str_attacker_id, 'CCSPlayer.m_iAccount')
            if int_player_money > 16000:
               int_player_money = 16000
            es.setplayerprop(str_attacker_id, 'CCSPlayer.m_iAccount', int_player_money)

      if (int_hitgroup_percent * int_weapon_percent) < 0:
         int_new_health = int(event_var['es_attackerhealth']) + ((int(event_var['dmg_health']) * int_hitgroup_percent * int_weapon_percent) / 10000)
         if int_new_health < 1:
            int_new_health = 1
            if int(es.ServerVar('regenmod_regeneration_cankill')):
               es.sexec(str_attacker_id, 'kill')
         es.setplayerprop(str_attacker_id, 'CBasePlayer.m_iHealth', int_new_health)
         int_new_armor = int(event_var['es_attackerarmor']) + ((int(event_var['dmg_armor']) * int_hitgroup_percent * int_weapon_percent) / 10000)
         if int_new_armor < 0:
            int_new_armor = 0
         es.setplayerprop(str_attacker_id, 'CCSPlayer.m_ArmorValue', int_new_armor)

      else:
         int_max_health = int(es.ServerVar('regenmod_regeneration_maxhealth'))
         if int(event_var['es_attackerhealth']) < int_max_health:
            int_new_health = int(event_var['es_attackerhealth']) + ((int(event_var['dmg_health']) * int_hitgroup_percent * int_weapon_percent) / 10000)
            if int_new_health > int_max_health:
               int_new_health = int_max_health
            es.setplayerprop(str_attacker_id, 'CBasePlayer.m_iHealth', int_new_health)
         int_max_armor = int(es.ServerVar('regenmod_regeneration_maxarmor'))
         if int(event_var['es_attackerarmor']) < int_max_armor:
            int_new_armor = int(event_var['es_attackerarmor']) + ((int(event_var['dmg_armor']) * int_hitgroup_percent * int_weapon_percent) / 10000)
            if int_new_armor > int_max_armor:
               int_new_armor = int_max_armor
            es.setplayerprop(str_attacker_id, 'CCSPlayer.m_ArmorValue', int_new_armor)


def player_blind(event_var):
   """Counts blinded players"""
   global int_blinded_players

   str_team = event_var['es_userteam']
   if str_team == '2':
      int_blinded_players -= 1
   elif str_team == '3':
      int_blinded_players += 1


def player_death(event_var):
   """
   Increases player kill and death count as necessary
   Increases the dead player's time alive
   """
   global dict_players

   dict_current_player = dict_players[event_var['userid']]
   dict_current_player['int_deaths'] += 1
   dict_current_player['float_time_alive'] += time.time() - dict_current_player['float_spawn_time']
   dict_current_player['float_spawn_time'] = 0
   if int(event_var['attacker']) and event_var['es_userteam'] <> event_var['es_attackerteam']:
      dict_players[event_var['attacker']]['int_kills'] += 1


def player_disconnect(event_var):
   """Removes the disconnecting player from dict_players"""
   if dict_players.has_key(event_var['userid']):
      del dict_players[event_var['userid']]


def hegrenade_detonate(event_var):
   """
   Determines the number of hegrenades the player should receive
   Gives the player the necessary number of hegreandes
   Sets current player's he damage count to 0
   """
   str_userid = event_var['userid']
   dict_current_player = dict_players[str_userid]

   if not int(event_var['es_userdead']):
      int_ammo_amount = es.getplayerprop(str_userid, dict_weapons['hegrenade']['prop'])
      int_ammo_option = int(es.ServerVar('regenmod_ammunition_hegrenades'))
      if not int_ammo_option:
         dict_current_player['int_hedamaged_players'] = 0
      if int_ammo_option > 1 and dict_current_player['int_hedamaged_players'] < 1:
         dict_current_player['int_hedamaged_players'] = 1

      if dict_current_player['int_hedamaged_players'] > 0:
         if not int_ammo_amount:
            es.give(str_userid, 'weapon_hegrenade')
         int_ammo_amount += dict_current_player['int_hedamaged_players']
         int_ammo_max = int(es.ServerVar('ammo_hegrenade_max'))
         if int_ammo_amount > int_ammo_max and not int(es.ServerVar('regenmod_ammunition_regenovermax')):
            int_ammo_amount = int_ammo_max
         es.setplayerprop(str_userid, dict_weapons['hegrenade']['prop'], int_ammo_amount)

   dict_current_player['int_hedamaged_players'] = 0


def flashbang_detonate(event_var):
   """
   Determines the number of flashbangs the player should receive
   Gives the player the necessary number of flashbangs
   Sets int_blinded_players to 0
   """
   global int_blinded_players

   if not int(event_var['es_userdead']):
      str_userid = event_var['userid']
      int_ammo_option = int(es.ServerVar('regenmod_ammunition_flashbangs'))
      if not int_ammo_option:
         int_blinded_players = 0
      elif event_var['es_userteam'] == '3':
         int_blinded_players *= -1
      int_ammo_amount = es.getplayerprop(str_userid, dict_weapons['flashbang']['prop'])
      if int_ammo_option > 1 and int_blinded_players < 1:
         int_blinded_players = 1

      if int_blinded_players > 0:
         if not int_ammo_amount:
            es.give(str_userid, 'weapon_flashbang')
         int_ammo_amount += int_blinded_players
         int_ammo_max = int(es.ServerVar('ammo_flashbang_max'))
         if int_ammo_amount > int_ammo_max and not int(es.ServerVar('regenmod_ammunition_regenovermax')):
            int_ammo_amount = int_ammo_max
         es.setplayerprop(str_userid, dict_weapons['flashbang']['prop'], int_ammo_amount)

   int_blinded_players = 0


def smokegrenade_detonate(event_var):
   """Gives the player a smoke grenade when an old smoke grenade detonates if necessay"""
   if int(es.ServerVar('regenmod_ammunition_smokegrenades')) and not int(event_var['es_userdead']):
      str_userid = event_var['userid']
      str_prop = dict_weapons['smokegrenade']['prop']
      int_ammo_amount = es.getplayerprop(str_userid, str_prop)
      if not int_ammo_amount:
         es.give(str_userid, 'weapon_smokegrenade')
      es.setplayerprop(str_userid, str_prop, int_ammo_amount + 1)


def unload():
   for str_option in dict_options:
      es.ServerVar(str_option).set(0)


def weapon_specific_cmd():
   """
   regenmod_regeneration_add <weapon> <percent>
   regenmod_regeneration_remove <weapon>
   Adds or removes weapon-specific regeneration
   """
   global dict_weapon_regen

   str_command_name = es.getargv(0).lower()
   if es.getargc() == {'regenmod_regeneration_add':3, 'regenmod_regeneration_remove':2}[str_command_name]:
      str_weapon = es.getargv(1).lower()
      if str_weapon[0:7] == 'weapon_':
         str_weapon = str_weapon[7:]
      if dict_weapons.has_key(str_weapon):

         if str_command_name == 'regenmod_regeneration_add':
            dict_weapon_regen[str_weapon] = int(es.getargv(2))
         elif dict_weapon_regen.has_key(str_weapon):
            del dict_weapon_regen[str_weapon]

      else:
         es.dbgmsg(0, 'RegenMod: Unrecognized weapon type \"%s\"' % str_weapon)

   else:
      es.dbgmsg(0, 'Syntax: ' + {'regenmod_regeneration_add':'regenmod_regeneration_add <weapon> <regeneration percent>', 'regenmod_regeneration_remove':'regenmod_regeneration_remove <weapon>'}[str_command_name])


def format_time(float_time):
   """Formats the float argument to a string in H:MM:SS or M:SS form"""
   int_hours = int(float_time / 3600)
   int_minutes = int(float_time % 3600 / 60)
   int_seconds = int(float_time % 60)
   if int_hours:
      return '%d:%02d:%02d' % (int_hours, int_minutes, int_seconds)
   return '%01d:%02d' % (int_minutes, int_seconds)


def hud_hint(str_userid, str_message):
   """Sends the player a HUD hint"""
   if es.exists('userid', str_userid):
      es.usermsg('create', 'regenmod_hud', 'HintText')
      es.usermsg('write', 'short', 'regenmod_hud', -1)
      es.usermsg('write', 'string', 'regenmod_hud', str_message)
      es.usermsg('send', 'regenmod_hud', str_userid)
      es.usermsg('delete', 'regenmod_hud')


def remove_tags(str_text):
   """Removes #lightgreen, #green and #default tags from the supplied string"""
   return str_text.replace('#lightgreen', '').replace('#green', '').replace('#default', '')


def missing_lang(str_heading, dict_tokens, str_language):
   """Displays an error message because the languages .ini could not be found"""
   return 'RegenMod: regenmod_languages.ini not found'